@@ -40,7 +40,7 @@
 
 #include "dejitter.h"
 
-static char const ident[]="$Id: dejitter.c 534 2015-12-16 17:49:00Z hb9xar $";
+static char const ident[]="$Id: dejitter.c 583 2019-11-09 17:41:08Z hb9xar $";
 
 #ifdef GPL
 
@@ -417,7 +417,7 @@ static void send_top_of_que(int nolock) {
             sts = rtp_relay_stop_fwd(&callid, 
                                      m->errret->direction,
                                      m->errret->media_stream_no,
-                                     nolock);
+                                     -1, nolock);
             if (sts != STS_SUCCESS) {
                /* force the streams to timeout on next occasion */
                m->errret->timestamp=0;
