@@ -28,6 +28,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
+#include <sys/time.h>
 #include <syslog.h>
 
 #include <errno.h>
@@ -39,7 +40,7 @@
 
 #include <signal.h>
 
-static char const ident[]="$Id: log.c 415 2009-02-28 22:02:11Z hb9xar $";
+static char const ident[]="$Id: log.c 587 2020-07-26 09:40:23Z hb9xar $";
 
 /* static functions */
 static void output_to_stderr(const char *label, va_list ap, char *file,
@@ -73,10 +74,17 @@ static int silence_level=1;
  *
  * use a 'fast' mutex for synchronizing - as these are portable... 
  */
-static pthread_mutex_t log_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t log_mutex; //&&& = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutexattr_t log_mutex_attr;
 
 void log_init(void) {
-   openlog(NULL,LOG_NDELAY|LOG_PID,LOG_DAEMON);
+   openlog("siproxd", LOG_NDELAY|LOG_PID, LOG_DAEMON);
+   pthread_mutexattr_init(&log_mutex_attr);
+   // need a recussive mutex, as logging itself does call ERROR/WARN/INFO while
+   // the mutex is locked by this thread.
+   // Otherwise we may end in a deadlock.
+   pthread_mutexattr_settype(&log_mutex_attr, PTHREAD_MUTEX_RECURSIVE);
+   pthread_mutex_init(&log_mutex, &log_mutex_attr);
 }
 
 void log_end(void) {
@@ -186,14 +194,15 @@ void log_tcp_connect(void) {
    sts=select(debug_listen_fd+1, &fdset, NULL, NULL, &timeout);
    if (sts > 0) {
       if (debug_fd != 0) {
-         tmpfd=accept(debug_listen_fd, NULL, NULL);
-         close(tmpfd);
-         INFO("Rejected DEBUG TCP connection");
+         while ((tmpfd=accept(debug_listen_fd, NULL, NULL)>=0)) {
+            close(tmpfd);
+            INFO("Rejected DEBUG TCP connection");
+         }
       } else {
          debug_fd=accept(debug_listen_fd, NULL, NULL);
          INFO("Accepted DEBUG TCP connection [fd=%i], debugpattern=%i",
               debug_fd, debug_pattern);
-         INFO(PACKAGE"-"VERSION"-"BUILDSTR" "UNAME);
+         INFO(PACKAGE"-"VERSION"-"BUILDSTR" "BUILDDATE" "UNAME);
       }
    }
 
@@ -224,15 +233,22 @@ void log_tcp_connect(void) {
 static void output_to_stderr(const char *label, va_list ap, char *file,
                              int line, const char *format) {
    va_list ap_copy;
-   time_t t;
+   int sts;
+   struct timeval tv;
    struct tm *tim;
 
    if (!log_to_stderr) return;
 
-   time(&t);
-   tim = localtime(&t);
-   fprintf(stderr, "%2.2i:%2.2i:%2.2i %s%s:%i ", tim->tm_hour,
-           tim->tm_min, tim->tm_sec, label, file, line);
+   sts = gettimeofday(&tv, NULL);
+   if (sts != 0) {
+      tv.tv_sec=0;
+      tv.tv_usec=0;
+   }
+
+   tim = localtime(&(tv.tv_sec));
+   fprintf(stderr, "%2.2i:%2.2i:%2.2i.%3.3i %s%s:%i ", 
+           tim->tm_hour, tim->tm_min, tim->tm_sec, 
+           (int)(tv.tv_usec/1000), label, file, line);
    va_copy(ap_copy, ap);
    vfprintf(stderr, format, ap_copy);
    va_end(ap_copy);
@@ -249,31 +265,51 @@ static void output_to_syslog(const char *label, int level, va_list ap,
    va_copy(ap_copy, ap);
    vsnprintf(outbuf, sizeof(outbuf), format, ap_copy);
    va_end(ap_copy);
-   syslog(LOG_USER|level, "%s:%i %s%s", file, line, label, outbuf);
+   syslog(LOG_DAEMON|level, "%s:%i %s%s", file, line, label, outbuf);
    return;
 }
 
 static void output_to_TCP(const char *label, va_list ap, char *file,
                           int line, const char *format) {
    va_list ap_copy;
-   time_t t;
+   int sts;
+   struct timeval tv;
    struct tm *tim;
    char outbuf[256];
-   int sts;
 
    if (debug_fd <= 0) return;
 
-   time(&t);
-   tim=localtime(&t);
-   snprintf(outbuf, sizeof(outbuf), "%2.2i:%2.2i:%2.2i %s%s:%i ",
-            tim->tm_hour, tim->tm_min, tim->tm_sec, label, file, line);
+   sts = gettimeofday(&tv, NULL);
+   if (sts != 0) {
+      tv.tv_sec=0;
+      tv.tv_usec=0;
+   }
+   tim = localtime(&(tv.tv_sec));
+   snprintf(outbuf, sizeof(outbuf), "%2.2i:%2.2i:%2.2i.%3.3i %s%s:%i ",
+            tim->tm_hour, tim->tm_min, tim->tm_sec, 
+            (int)(tv.tv_usec/1000), label, file, line);
    sts=write(debug_fd, outbuf, strlen(outbuf));
+   if (sts < 0) {
+      // assume TCP debug connection has been aborted in some bad way
+      close(debug_fd);
+      debug_fd=0;
+   }
    va_copy(ap_copy, ap);
    vsnprintf(outbuf, sizeof(outbuf), format, ap_copy);
    va_end(ap_copy);
    sts=write(debug_fd, outbuf, strlen(outbuf));
+   if (sts < 0) {
+      // assume TCP debug connection has been aborted in some bad way
+      close(debug_fd);
+      debug_fd=0;
+   }
    snprintf(outbuf, sizeof(outbuf), "\n");
    sts=write(debug_fd, outbuf, strlen(outbuf));
+   if (sts < 0) {
+      // assume TCP debug connection has been aborted in some bad way
+      close(debug_fd);
+      debug_fd=0;
+   }
    return;
 }
 
@@ -288,7 +324,7 @@ void log_debug(unsigned int class, char *file, int line, const char *format, ...
 
    output_to_stderr("", ap, file, line, format);
 
-   if (!log_to_stderr && silence_level < 1) {
+   if (silence_level < 1) {
       output_to_syslog("", LOG_DEBUG, ap, file, line, format);
    }
 
@@ -370,6 +406,12 @@ void log_dump_buffer(unsigned int class, char *file, int line,
    if (debug_fd > 0) {
       snprintf(outbuf, sizeof(outbuf) ,"---BUFFER DUMP follows---\n");
       sts=write(debug_fd, outbuf, strlen(outbuf));
+      if (sts < 0) {
+         ERROR("write returned error [%i:%s]",errno, strerror(errno));
+         // assume TCP debug connection has been aborted in some bad way
+         close(debug_fd);
+         debug_fd=0;
+      }
    }
 
    for (i=0; i<length; i+=16) {
@@ -388,6 +430,12 @@ void log_dump_buffer(unsigned int class, char *file, int line,
          snprintf(outbuf, sizeof(outbuf) ,"  %-47.47s %-16.16s\n",
                   tmplin1, tmplin2);
          sts=write(debug_fd, outbuf, strlen(outbuf));
+         if (sts < 0) {
+            ERROR("write returned error [%i:%s]",errno, strerror(errno));
+            // assume TCP debug connection has been aborted in some bad way
+            close(debug_fd);
+            debug_fd=0;
+         }
       }
    }
 
@@ -398,6 +446,12 @@ void log_dump_buffer(unsigned int class, char *file, int line,
    if (debug_fd > 0) {
       snprintf(outbuf, sizeof(outbuf) ,"---end of BUFFER DUMP---\n");
       sts=write(debug_fd, outbuf, strlen(outbuf));
+      if (sts < 0) {
+         ERROR("write returned error [%i:%s]",errno, strerror(errno));
+         // assume TCP debug connection has been aborted in some bad way
+         close(debug_fd);
+         debug_fd=0;
+      }
    }
    pthread_mutex_unlock(&log_mutex);
 
