@@ -45,7 +45,7 @@
 #include "redirect_cache.h"
 #include "log.h"
 
-static char const ident[]="$Id: plugin_regex.c 526 2015-09-19 14:54:42Z hb9xar $";
+static char const ident[]="$Id: plugin_regex.c 557 2017-02-27 20:15:51Z hb9xar $";
 
 /* Plug-in identification */
 static char name[]="plugin_regex";
@@ -117,6 +117,15 @@ int  PLUGIN_PROCESS(int stage, sip_ticket_t *ticket){
 
 /* De-Initialization */
 int  PLUGIN_END(plugin_def_t *plugin_def){
+   int i;
+   int num_entries;
+
+   /* free space for regexes */
+   num_entries = plugin_cfg.regex_pattern.used;
+   for (i=0; i < num_entries; i++) {
+      regfree(&re[i]);
+   }
+   free(re);
    return STS_SUCCESS;
 }
 
