@@ -18,15 +18,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */
 
-/* $Id: plugins.h 532 2015-10-15 08:38:20Z hb9xar $ */
+/* $Id: plugins.h 567 2017-05-28 13:21:53Z hb9xar $ */
 
-#ifdef LTDL_CONVLIB
-   /* fetch local version of ltdl */
-   #include "../libltdl/ltdl.h"
-#else
-   /* fetch system version of libltdl */
-   #include <ltdl.h>
-#endif
+#include <ltdl.h>
 
 #ifdef WITH_LTDL_FIX
 /* workaround for some broken libtool 2.2.6 (and others?) versions 
@@ -35,16 +29,20 @@
  * error during linking state */
 #ifndef lt__PROGRAM__LTX_preloaded_symbols
 #define lt__PROGRAM__LTX_preloaded_symbols lt_libltdl_LTX_preloaded_symbols
-//extern const void *lt_preloaded_symbols[];
 #endif
 #endif 
 
+
 /* Plugins must return STS_SUCCESS / SUCCESS_FAILURE */
 
 
 /*
  * Processing stages for Plugins
  */
+/* NOOP */
+/* No activation */
+#define PLUGIN_NOOP		0x00000000
+
 /* get cyclic trigger */
 /* NO ticket is present (ticket = NULL pointer) */
 #define PLUGIN_TIMER		0x00000001
@@ -53,12 +51,12 @@
 /* may end the current SIP processing in siproxd by returning STS_FALSE *
  * may be used to intercept other traffic on SIP port */
 /* ticket with NO sipmsg is present (ticket.sipmsg = NULL pointer) */
-#define PLUGIN_PROCESS_RAW	0x00000005
+#define PLUGIN_PROCESS_RAW	0x00000008
 
 /*--------- below here a valid sip message (ticket->sipmsg) is present ---*/
 
 /* Validation of SIP packet */
-/* may end the current SIP processing in siproxd by returning STS_FALSE *
+/* may end the current SIP processing in siproxd by returning STS_FALSE/STS_FAILURE *
  * may be used to intercept other traffic on SIP port */
 #define PLUGIN_VALIDATE		0x00000010	
 
@@ -91,7 +89,7 @@ typedef struct {
    lt_ptr dlhandle;	/* handle returned by dlopen() */
 } plugin_def_t;
 
-#define SIPROXD_API_VERSION	0x0101
+#define SIPROXD_API_VERSION	0x0102
 
 
 /* The plugin must provide the following entry points */
