@@ -37,7 +37,7 @@
 #include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: proxy.c 528 2015-09-19 22:32:55Z hb9xar $";
+static char const ident[]="$Id: proxy.c 583 2019-11-09 17:41:08Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;	/* defined in siproxd.c */
@@ -148,8 +148,8 @@ DEBUGC(DBCLASS_PROXY,"index i=%i",i);
       /* if this is CANCEL/BYE request, stop RTP proxying */
       if (MSG_IS_BYE(request) || MSG_IS_CANCEL(request)) {
          /* stop the RTP proxying stream(s) */
-         rtp_stop_fwd(osip_message_get_call_id(request), DIR_INCOMING);
-         rtp_stop_fwd(osip_message_get_call_id(request), DIR_OUTGOING);
+         rtp_stop_fwd(osip_message_get_call_id(request), DIR_INCOMING, -1);
+         rtp_stop_fwd(osip_message_get_call_id(request), DIR_OUTGOING, -1);
 
       /* check for incoming request */
       } else if (MSG_IS_INVITE(request)) {
@@ -221,8 +221,8 @@ sts=sip_obscure_callid(ticket);
       /* if this is CANCEL/BYE request, stop RTP proxying */
       if (MSG_IS_BYE(request) || MSG_IS_CANCEL(request)) {
          /* stop the RTP proxying stream(s) */
-         rtp_stop_fwd(osip_message_get_call_id(request), DIR_INCOMING);
-         rtp_stop_fwd(osip_message_get_call_id(request), DIR_OUTGOING);
+         rtp_stop_fwd(osip_message_get_call_id(request), DIR_INCOMING, -1);
+         rtp_stop_fwd(osip_message_get_call_id(request), DIR_OUTGOING, -1);
       }
 
 sts=sip_obscure_callid(ticket);
@@ -382,11 +382,15 @@ sts=sip_obscure_callid(ticket);
     */
    } else {
       /* get the destination from the SIP URI */
+
+      /* 1) try SRV record */
 /*&&&& Here, the SRV record lookup magic must go.
 In a first implementation we may just try to get the lowest priority,
 max weighted '_sip._udp.domain' entry and port number.
 No load balancing and no failover are supported with this.
 &&&*/
+
+      /* 2) if no SRV record found, resolve A record */
       sts = get_ip_by_host(request->req_uri->host, &ticket->next_hop.sin_addr);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_request: cannot resolve URI [%s]",
@@ -483,6 +487,7 @@ int proxy_response (sip_ticket_t *ticket) {
    char *buffer;
    size_t buflen;
    osip_message_t *response;
+   int cseq;
 
    DEBUGC(DBCLASS_PROXY,"proxy_response");
 
@@ -546,8 +551,24 @@ sts=sip_obscure_callid(ticket);
          } else if ((MSG_IS_STATUS_4XX(response))  ||
                      (MSG_IS_STATUS_5XX(response)) ||
                      (MSG_IS_STATUS_6XX(response))) {
-            rtp_stop_fwd(osip_message_get_call_id(response), DIR_INCOMING);
-            rtp_stop_fwd(osip_message_get_call_id(response), DIR_OUTGOING);
+//&&& in case of repetitions, this logic breaks!
+// INVITE w/o auth credentials
+// repeated INVITE w/o auth credentials
+// 407 response to 1st INVITE
+// INVITE w/ auth credentials
+// 407 response to 2nd INVITE
+// -> Bzzz, siproxd cancels the set-up RTP ports from INVITE(3) above
+// I could:
+// - simply ingore these stati (rtp relay would time-out eventually)
+// - reduce rtp timeout for this stream and have if time-out quicker
+// - somehow remember the CSEQ number of the last request and only do
+//   something if this CSEQ is equal (or higher) to the last request that changed 
+//   the RTP relay setup
+//      tmp = osip_cseq_get_number(sip_msg->cseq);
+            cseq = atoi(osip_cseq_get_number(response->cseq));
+
+            rtp_stop_fwd(osip_message_get_call_id(response), DIR_INCOMING, cseq);
+            rtp_stop_fwd(osip_message_get_call_id(response), DIR_OUTGOING, cseq);
          }
       } /* if INVITE */
 
@@ -559,7 +580,7 @@ sts=sip_obscure_callid(ticket);
           * REGISTER returns *my* Contact header information.
           * Rewrite Contact header back to represent the true address.
           * Other responses do return the Contact header of the sender.
-          * also change the expiration timeout to the value returned by the
+          * Also change the expiration timeout to the value returned by the
           * server.
           */
          sts = register_set_expire(ticket);
@@ -633,8 +654,11 @@ sts=sip_obscure_callid(ticket);
          } else if ((MSG_IS_STATUS_4XX(response))  ||
                      (MSG_IS_STATUS_5XX(response)) ||
                      (MSG_IS_STATUS_6XX(response))) {
-            rtp_stop_fwd(osip_message_get_call_id(response), DIR_INCOMING);
-            rtp_stop_fwd(osip_message_get_call_id(response), DIR_OUTGOING);
+//&&& same here, repetitions break this logic
+            cseq = atoi(osip_cseq_get_number(response->cseq));
+
+            rtp_stop_fwd(osip_message_get_call_id(response), DIR_INCOMING, cseq);
+            rtp_stop_fwd(osip_message_get_call_id(response), DIR_OUTGOING, cseq);
          }
       } /* if INVITE */
 
@@ -778,6 +802,7 @@ int proxy_rewrite_invitation_body(sip_ticket_t *ticket, int direction){
    int call_direction=0;
    int have_c_media=0;
    int isrtp = 0 ;
+   int cseq = 0;
 
    if (configuration.rtp_proxy_enable == 0) return STS_SUCCESS;
 
@@ -1112,13 +1137,14 @@ if (configuration.debuglevel)
             /*
              * Start the RTP stream
              */
+            cseq = atoi(osip_cseq_get_number(mymsg->cseq));
             sts = rtp_start_fwd(osip_message_get_call_id(mymsg),
                                 client_id,
                                 rtp_direction, call_direction,
                                 media_stream_no,
                                 map_addr, &map_port,
                                 addr_media, msg_port,
-                                isrtp);
+                                isrtp, cseq);
 
             if (sts == STS_SUCCESS) {
                /* and rewrite the port */
@@ -1213,6 +1239,8 @@ int proxy_rewrite_request_uri(osip_message_t *mymsg, int idx){
    osip_uri_to_str(url, &tmp1);
    osip_uri_to_str(urlmap[idx].true_url, &tmp2);
    DEBUGC(DBCLASS_BABBLE,"proxy_rewrite_request_uri: %s -> %s", tmp1, tmp2);
+   if (tmp1) osip_free(tmp1);
+   if (tmp2) osip_free(tmp2);
 
    osip_uri_free(url);
    url=NULL;
