@@ -46,7 +46,7 @@
    #include "dejitter.h"
 #endif
 
-static char const ident[]="$Id: rtpproxy_relay.c 482 2011-06-12 18:45:17Z hb9xar $";
+static char const ident[]="$Id: rtpproxy_relay.c 583 2019-11-09 17:41:08Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
@@ -118,7 +118,7 @@ int rtp_relay_init( void ) {
    pthread_attr_init(&attr);
    pthread_attr_init(&attr);
    pthread_attr_getstacksize (&attr, &stacksize);
-   INFO("Current thread stacksize is %i kB",stacksize/1024);   
+   INFO("Current thread stacksize is %u kB",(unsigned int)(stacksize/1024));
 
    /* experimental feature:
     * reduce the thread stack size to reduce the overall
@@ -132,7 +132,7 @@ int rtp_relay_init( void ) {
    if (configuration.thread_stack_size > 0) {
       stacksize = configuration.thread_stack_size*1024;
       pthread_attr_setstacksize (&attr, stacksize);
-      INFO("Setting new thread stacksize to %i kB",stacksize/1024);   
+      INFO("Setting new thread stacksize to %u kB",(unsigned int)stacksize/1024);
    }
 
    DEBUGC(DBCLASS_RTP,"create thread");
@@ -391,7 +391,7 @@ static void *rtpproxy_main(void *arg) {
                            sts = rtp_relay_stop_fwd(&callid,
                                                     rtp_proxytable[i].direction,
                                                     rtp_proxytable[i].media_stream_no,
-                                                    NOLOCK_FDSET);
+                                                    -1, NOLOCK_FDSET);
                            if (sts != STS_SUCCESS) {
                               /* force the streams to timeout on next occasion */
                               rtp_proxytable[i].timestamp=0;
@@ -426,7 +426,7 @@ static void *rtpproxy_main(void *arg) {
                         sts = rtp_relay_stop_fwd(&callid,
                                                  rtp_proxytable[i].direction,
                                                  rtp_proxytable[i].media_stream_no,
-                                                 NOLOCK_FDSET);
+                                                 -1, NOLOCK_FDSET);
                         if (sts != STS_SUCCESS) {
                            /* force the streams to timeout on next occasion */
                            rtp_proxytable[i].timestamp=0;
@@ -441,8 +441,8 @@ static void *rtpproxy_main(void *arg) {
              * This allows silence (no data) on one direction without breaking
              * the connection after the RTP timeout */
             rtp_proxytable[i].timestamp=current_tv.tv_sec;
-            if (rtp_proxytable[i].opposite_entry > 0) {
-               rtp_proxytable[rtp_proxytable[i].opposite_entry-1].timestamp=
+            if (rtp_proxytable[i].opposite_entry >= 0) {
+               rtp_proxytable[rtp_proxytable[i].opposite_entry].timestamp=
                   current_tv.tv_sec;
             }
          } /* if */
@@ -482,7 +482,7 @@ static void *rtpproxy_main(void *arg) {
                 * just one (unused?) has timed out. Seen with VoIPEX PBX! */
                rtp_relay_stop_fwd(&callid, rtp_proxytable[i].direction,
                                   rtp_proxytable[i].media_stream_no,
-                                  NOLOCK_FDSET);
+                                  -1, NOLOCK_FDSET);
             } /* if */
          } /* for i */
       } /* if (t>...) */
@@ -512,7 +512,7 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
                          int rtp_direction, int call_direction,
                          int media_stream_no, struct in_addr local_ipaddr,
                          int *local_port, struct in_addr remote_ipaddr,
-                         int remote_port, int dejitter) {
+                         int remote_port, int dejitter, int cseq) {
    static int prev_used_port = 0;
    int num_ports;
    int i2, i, j;
@@ -548,11 +548,12 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
    }
 
    DEBUGC(DBCLASS_RTP,"rtp_relay_start_fwd: starting RTP proxy "
-          "stream for: CallID=%s@%s [Client-ID=%s] (%s,%s) #=%i",
+          "stream for: CallID=%s@%s [Client-ID=%s] (%s,%s) "
+          "cseq=%i #=%i",
           callid->number, callid->host, client_id.idstring,
           ((rtp_direction == DIR_INCOMING) ? "incoming RTP" : "outgoing RTP"),
           ((call_direction == DIR_INCOMING) ? "incoming Call" : "outgoing Call"),
-          media_stream_no);
+          cseq, media_stream_no);
 
    /* lock mutex */
    #define return is_forbidden_in_this_code_section
@@ -603,6 +604,12 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
                      sizeof(remote_ipaddr));
          }
 
+         /* update CSEQ in proxytable if the current request has a higher one */
+         if (cseq > rtp_proxytable[i].cseq) {
+            rtp_proxytable[i].cseq = cseq;
+         }
+
+
 #ifdef USE_DEJITTER
          /* Initialize up timecrontrol for dejitter function */
          if ((configuration.rtp_input_dejitter > 0) || 
@@ -614,11 +621,12 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
 
          /* return the already known local port number */
          DEBUGC(DBCLASS_RTP,"RTP stream already active idx=%i (remaddr=%s, "
-                "remport=%i, lclport=%i, id=%s, #=%i)",
+                "remport=%i, lclport=%i, id=%s, cseq=%i, #=%i)",
                 i, utils_inet_ntoa(remote_ipaddr),
                 rtp_proxytable[i].remote_port,
                 rtp_proxytable[i].local_port,
                 rtp_proxytable[i].callid_number,
+                rtp_proxytable[i].cseq,
                 rtp_proxytable[i].media_stream_no);
          *local_port=rtp_proxytable[i].local_port;
          sts = STS_SUCCESS;
@@ -759,6 +767,7 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
    /* store the passed Client-ID data */
    memcpy(&rtp_proxytable[freeidx].client_id, &client_id, sizeof(client_id_t));
 
+   rtp_proxytable[freeidx].cseq = cseq;
    rtp_proxytable[freeidx].direction = rtp_direction;
    rtp_proxytable[freeidx].call_direction = call_direction;
    rtp_proxytable[freeidx].media_stream_no = media_stream_no;
@@ -807,12 +816,13 @@ int rtp_relay_start_fwd (osip_call_id_t *callid, client_id_t client_id,
 
 //&&&
    DEBUGC(DBCLASS_RTP,"rtp_relay_start_fwd: started RTP proxy "
-          "stream for: CallID=%s@%s [Client-ID=%s] %s #=%i idx=%i",
+          "stream for: CallID=%s@%s [Client-ID=%s] %s cseq=%i, "
+          "#=%i idx=%i",
           rtp_proxytable[freeidx].callid_number,
           rtp_proxytable[freeidx].callid_host,
           rtp_proxytable[freeidx].client_id.idstring,
           ((rtp_proxytable[freeidx].direction == DIR_INCOMING) ? "incoming RTP" : "outgoing RTP"),
-          rtp_proxytable[freeidx].media_stream_no, freeidx);
+          cseq, rtp_proxytable[freeidx].media_stream_no, freeidx);
 
 unlock_and_exit:
    /* unlock mutex */
@@ -829,13 +839,15 @@ unlock_and_exit:
  * if media_stream_no == -1, all media streams will be stopped,
  * otherwise only the specified one.
  *
+ * if cseq == -1, it will be ignored.
+ *
  * RETURNS
  *	STS_SUCCESS on success
  *	STS_FAILURE on error
  */
 int rtp_relay_stop_fwd (osip_call_id_t *callid,
                         int rtp_direction,
-                        int media_stream_no, int nolock) {
+                        int media_stream_no, int cseq, int nolock) {
    int i, sts;
    int retsts=STS_SUCCESS;
    int got_match=0;
@@ -847,10 +859,10 @@ int rtp_relay_stop_fwd (osip_call_id_t *callid,
    }
 
    DEBUGC(DBCLASS_RTP,"rtp_relay_stop_fwd: stopping RTP proxy "
-          "stream for: %s@%s (%s) (nolock=%i)",
+          "stream for: %s@%s (%s), cseq=%i (nolock=%i)",
           callid->number, callid->host,
           ((rtp_direction == DIR_INCOMING) ? "incoming" : "outgoing"),
-          nolock);
+          cseq, nolock);
 
    /*
     * lock mutex - only if not requested to skip the lock.
@@ -892,9 +904,17 @@ int rtp_relay_stop_fwd (osip_call_id_t *callid,
          (compare_callid(callid, &cid) == STS_SUCCESS) &&
          (rtp_proxytable[i].direction == rtp_direction) &&
          ((media_stream_no < 0) ||
-          (media_stream_no == rtp_proxytable[i].media_stream_no))) {
+          (media_stream_no == rtp_proxytable[i].media_stream_no)) &&
+         ((cseq < 0) ||
+          (cseq >= rtp_proxytable[i].cseq))
+         ) {
+
          /* close RTP sockets */
-         sts = close(rtp_proxytable[i].rtp_rx_sock);
+         if (rtp_proxytable[i].rtp_rx_sock > 0) {
+            sts = close(rtp_proxytable[i].rtp_rx_sock);
+         } else {
+            sts=0;
+         }
          DEBUGC(DBCLASS_RTP,"closed socket %i for RTP stream "
                 "%s:%s == %s:%s  (idx=%i) sts=%i",
                 rtp_proxytable[i].rtp_rx_sock,
@@ -914,7 +934,11 @@ int rtp_relay_stop_fwd (osip_call_id_t *callid,
                    rtp_proxytable[i].remote_ipaddr,
                    rtp_proxytable[i].remote_port);
          /* close RTCP socket */
-         sts = close(rtp_proxytable[i].rtp_con_rx_sock);
+         if (rtp_proxytable[i].rtp_con_rx_sock > 0) {
+            sts = close(rtp_proxytable[i].rtp_con_rx_sock);
+         } else {
+            sts=0;
+         }
          DEBUGC(DBCLASS_RTP,"closed socket %i for RTCP stream sts=%i",
                 rtp_proxytable[i].rtp_con_rx_sock, sts);
          if (sts < 0) {
@@ -930,8 +954,8 @@ int rtp_relay_stop_fwd (osip_call_id_t *callid,
                    rtp_proxytable[i].remote_ipaddr,
                    rtp_proxytable[i].remote_port + 1);
          /* clean up */
-         if (rtp_proxytable[i].opposite_entry) {
-            rtp_proxytable[rtp_proxytable[i].opposite_entry-1].opposite_entry=0;
+         if (rtp_proxytable[i].opposite_entry >= 0) {
+            rtp_proxytable[rtp_proxytable[i].opposite_entry].opposite_entry=-1;
          }
          memset(&rtp_proxytable[i], 0, sizeof(rtp_proxytable[0]));
          got_match=1;
@@ -1016,7 +1040,10 @@ static void rtpproxy_kill( void ) {
          cid.host   = rtp_proxytable[i].callid_host;
          sts = rtp_relay_stop_fwd(&cid, rtp_proxytable[i].direction,
                                   rtp_proxytable[i].media_stream_no,
-                                  LOCK_FDSET);
+                                  -1, LOCK_FDSET);
+         if (sts != STS_SUCCESS) {
+            DEBUGC(DBCLASS_RTP,"rtp_relay_stop_fwd did return error");
+         }
       }
    }
    
@@ -1078,8 +1105,8 @@ static int match_socket (int rtp_proxytable_idx) {
          strcpy(remip2, utils_inet_ntoa(rtp_proxytable[rtp_proxytable_idx].remote_ipaddr));
          strcpy(lclip2, utils_inet_ntoa(rtp_proxytable[rtp_proxytable_idx].local_ipaddr));
 
-         rtp_proxytable[rtp_proxytable_idx].opposite_entry=j+1;
-         rtp_proxytable[j].opposite_entry=rtp_proxytable_idx+1;
+         rtp_proxytable[rtp_proxytable_idx].opposite_entry=j;
+         rtp_proxytable[j].opposite_entry=rtp_proxytable_idx;
 
          DEBUGC(DBCLASS_RTP, "connected entry %i (fd=%i, %s:%i->%s:%i) <-> entry %i (fd=%i, %s:%i->%s:%i)",
                              j, rtp_proxytable[j].rtp_rx_sock,
