@@ -42,7 +42,7 @@
 #include "digcalc.h"
 #include "log.h"
 
-static char const ident[]="$Id: sip_utils.c 538 2016-02-24 19:14:23Z hb9xar $";
+static char const ident[]="$Id: sip_utils.c 585 2020-07-07 22:05:29Z hb9xar $";
 
 
 /* configuration storage */
@@ -290,19 +290,28 @@ int compare_url(osip_uri_t *url1, osip_uri_t *url2) {
       WARN("compare_url: NULL scheme - ignoring");
    }
 
-   /* compare username (if present) case sensitive */
+   /*
+    * first compare the username, avoid unneccesaery DNS lookups
+    * compare username
+    * - if present on both: must match case sensitive
+    * - if present only on one side -> failure (mismatch)
+    * - if not present (NULL) on both sides: -> match
+    */
    if (url1->username && url2->username) {
       if (strcmp(url1->username, url2->username) != 0) {
-//         DEBUGC(DBCLASS_BABBLE, "compare_url: username mismatch");
+//         DEBUGC(DBCLASS_BABBLE, "compare_url: username mismatch [%s <-> %s]",
+//                url1->username, url2->username);
          return STS_FAILURE;
       }
+   } else if (! ((url1->username==NULL) && (url2->username==NULL))) {
+//      DEBUGC(DBCLASS_BABBLE, "compare_url: username mismatch (NULL/not-NULL");
+      return STS_FAILURE;
    } else {
-//      DEBUGC(DBCLASS_BABBLE, "compare_url: NULL username - ignoring");
+//      DEBUGC(DBCLASS_BABBLE, "compare_url: both NULL username - consider it a match");
    }
 
-
    /*
-    * now, try to resolve the host. If resolveable, compare
+    * finally, try to resolve the host. If resolveable, compare
     * IP addresses - if not resolveable, compare the host names
     * itselfes
     */
@@ -323,13 +332,13 @@ int compare_url(osip_uri_t *url1, osip_uri_t *url2) {
    if ((sts1 == STS_SUCCESS) && (sts2 == STS_SUCCESS)) {
       /* compare IP addresses */
       if (memcmp(&addr1, &addr2, sizeof(addr1))!=0) {
-         DEBUGC(DBCLASS_BABBLE, "compare_url: IP mismatch");
+//         DEBUGC(DBCLASS_BABBLE, "compare_url: IP mismatch");
          return STS_FAILURE;
       }
    } else {
       /* compare hostname strings case INsensitive */
       if (osip_strcasecmp(url1->host, url2->host) != 0) {
-         DEBUGC(DBCLASS_BABBLE, "compare_url: host name mismatch");
+//         DEBUGC(DBCLASS_BABBLE, "compare_url: host name mismatch");
          return STS_FAILURE;
       }
    }
@@ -430,7 +439,7 @@ int is_sipuri_local (sip_ticket_t *ticket) {
       /* need name resolution */
       sts=get_ip_by_host(sip->req_uri->host, &addr_uri);
       if (sts == STS_FAILURE) {
-         DEBUGC(DBCLASS_PROXY, "sip_gen_response: cannot resolve request uri [%s]",
+         DEBUGC(DBCLASS_PROXY, "is_sipuri_local: cannot resolve request uri [%s]",
                 sip->req_uri->host);
          return STS_FALSE;
       }
@@ -615,7 +624,6 @@ int sip_add_myvia (sip_ticket_t *ticket, int interface) {
  */
 int sip_del_myvia (sip_ticket_t *ticket) {
    osip_via_t *via;
-   int sts;
 
    DEBUGC(DBCLASS_PROXY,"deleting topmost VIA");
    via = osip_list_get (&(ticket->sipmsg->vias), 0);
@@ -630,7 +638,7 @@ int sip_del_myvia (sip_ticket_t *ticket) {
       return STS_FAILURE;
    }
 
-   sts = osip_list_remove(&(ticket->sipmsg->vias), 0);
+   osip_list_remove(&(ticket->sipmsg->vias), 0);
    osip_via_free (via);
    return STS_SUCCESS;
 }
@@ -1049,6 +1057,9 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
     */
    for (i=0; i<URLMAP_SIZE; i++) {
       if (urlmap[i].active == 0) continue;
+      /* outgoing requests may include the grace period, do
+       * not filter for  urlmap[].expires */
+      
       if (get_ip_by_host(urlmap[i].true_url->host, &tmp_addr) == STS_FAILURE) {
          DEBUGC(DBCLASS_SIP, "sip_find_direction: cannot resolve host [%s]",
              urlmap[i].true_url->host);
@@ -1061,6 +1072,9 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
             } else {
                type=RESTYP_OUTGOING;
             }
+            DEBUGC(DBCLASS_SIP, "sip_find_direction: found internal client, "
+                   "type=%i, ip=%s",
+                   type, utils_inet_ntoa(from->sin_addr));
             break;
          }
       }
@@ -1078,6 +1092,11 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
    if (type == DIRTYP_UNKNOWN) {
       for (i=0; i<URLMAP_SIZE; i++) {
          if (urlmap[i].active == 0) continue;
+         /* an incoming REGISTER RESPONSE may be processed withing 
+          * the grace period, but no other incoming request/response */
+         if ((!MSG_IS_RESPONSE_FOR(ticket->sipmsg,"REGISTER")) &&
+            (urlmap[i].expires < ticket->timestamp)) continue;
+
          /* RFC3261:
           * 'To' contains a display name (Bob) and a SIP or SIPS URI
           * (sip:bob@biloxi.com) towards which the request was originally
@@ -1097,6 +1116,7 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
                 (!MSG_IS_REGISTER(request) &&
                  (compare_url(request->to->url, urlmap[i].reg_url)==STS_SUCCESS))) {
                type=REQTYP_INCOMING;
+               DEBUGC(DBCLASS_SIP, "sip_find_direction: found - incoming request");
                break;
             }
          } else { 
@@ -1105,13 +1125,14 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
             if ((compare_url(response->from->url, urlmap[i].reg_url)==STS_SUCCESS) ||
                 (compare_url(response->from->url, urlmap[i].masq_url)==STS_SUCCESS)) {
                type=RESTYP_INCOMING;
+               DEBUGC(DBCLASS_SIP, "sip_find_direction: found - incoming response");
                break;
             }
          } /* is request */
       } /* for i */
    } /* if type == DIRTYP_UNKNOWN */
    if (type == DIRTYP_UNKNOWN) {
-      DEBUGC(DBCLASS_SIP, "sip_find_direction: no INCOMING (To:) found");
+      DEBUGC(DBCLASS_SIP, "sip_find_direction: no INCOMING (To:/From:) found");
    }
 
 
@@ -1123,6 +1144,10 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
    if ((type == DIRTYP_UNKNOWN) && (MSG_IS_REQUEST(ticket->sipmsg))) {
       for (i=0; i<URLMAP_SIZE; i++) {
          if (urlmap[i].active == 0) continue;
+         /* an incoming REGISTER RESPONSE may be processed withing 
+          * the grace period, but no other incoming request/response */
+         if ((!MSG_IS_RESPONSE_FOR(ticket->sipmsg,"REGISTER")) &&
+            (urlmap[i].expires < ticket->timestamp)) continue;
          /* incoming request (SIP URI == 'masq') || ((SIP URI == 'reg') && !REGISTER)*/
          if ((compare_url(request->req_uri, urlmap[i].masq_url)==STS_SUCCESS) ||
              (!MSG_IS_REGISTER(request) &&
@@ -1174,6 +1199,10 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
          if (sts == STS_SUCCESS) {
             for (i=0; i<URLMAP_SIZE; i++) {
                if (urlmap[i].active == 0) continue;
+               /* an incoming REGISTER RESPONSE may be processed withing 
+                * the grace period, but no other incoming request/response */
+               if ((!MSG_IS_RESPONSE_FOR(ticket->sipmsg,"REGISTER")) &&
+                  (urlmap[i].expires < ticket->timestamp)) continue;
                /* incoming response (1st via in list points to a registered UA) */
                sts=get_ip_by_host(urlmap[i].true_url->host, &addr_myself);
                if (sts == STS_FAILURE) {
@@ -1250,11 +1279,17 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
    if (i < URLMAP_SIZE) {
       if (urlidx) *urlidx=i;
       DEBUGC(DBCLASS_SIP, "sip_find_direction: dir=%i, urlmap %i, "
-                          "trueurl [%s:%s] / masqurl [%s:%s] / regurl [%s:%s]",
+                          "trueurl [%s@%s:%s] / masqurl [%s@%s:%s] / regurl [%s@%s:%s]",
                           type, i,
-                          urlmap[i].true_url->host, urlmap[i].true_url->port,
-                          urlmap[i].masq_url->host, urlmap[i].masq_url->port,
-                          urlmap[i].reg_url->host, urlmap[i].reg_url->port);
+                          (urlmap[i].true_url->username)? urlmap[i].true_url->username : "(null)", 
+                          (urlmap[i].true_url->host)? urlmap[i].true_url->host : "(null)", 
+                          urlmap[i].true_url->port,
+                          (urlmap[i].masq_url->username)? urlmap[i].masq_url->username : "(null)", 
+                          (urlmap[i].masq_url->host)? urlmap[i].masq_url->host : "(null)", 
+                          urlmap[i].masq_url->port,
+                          (urlmap[i].reg_url->username)? urlmap[i].reg_url->username : "(null)", 
+                          (urlmap[i].reg_url->host)? urlmap[i].reg_url->host : "(null)", 
+                          urlmap[i].reg_url->port);
    } else {
       if (urlidx) *urlidx=-1;
       DEBUGC(DBCLASS_SIP, "sip_find_direction: dir=%i, not found in URLMAP", type);
