@@ -23,6 +23,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
+#include <time.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -40,7 +41,7 @@
 #include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: siproxd.c 524 2015-09-19 12:56:27Z hb9xar $";
+static char const ident[]="$Id: siproxd.c 587 2020-07-26 09:40:23Z hb9xar $";
 
 /* configuration storage */
 struct siproxd_config configuration;
@@ -98,7 +99,10 @@ FILE *siproxd_passwordfile;
 
 /* -h help option text */
 static const char str_helpmsg[] =
-PACKAGE "-" VERSION "-" BUILDSTR " (c) 2002-2011 Thomas Ries\n"
+PACKAGE "-" VERSION "-" BUILDSTR "\n" \
+"Build date: " BUILDDATE "\n" \
+"Plattform:  " UNAME "\n" \
+"(c) 2002-2016 Thomas Ries\n"
 "\nUsage: siproxd [options]\n\n"
 "options:\n"
 #ifdef  HAVE_GETOPT_LONG
@@ -146,8 +150,13 @@ int main (int argc, char *argv[])
    char *pidfilename=NULL;
    struct sigaction act;
 
+/*
+ * initialize the loggin subsystem
+ */
    log_init();
-
+   /* set log silencing for syslog as requested in config */
+   log_set_silence(configuration.silence_log);
+   /* while not daemonized, log also to stderr */
    log_set_stderr(1);
 
 /*
@@ -168,6 +177,9 @@ int main (int argc, char *argv[])
    if (sigaction(SIGPIPE, &act, NULL)) {
       ERROR("Failed to install SIGPIPE handler");
    }
+   if (sigaction(SIGHUP, &act, NULL)) {
+      ERROR("Failed to install SIGHUP handler");
+   }
 
 
 /*
@@ -230,7 +242,7 @@ int main (int argc, char *argv[])
 /*
  * Init stuff
  */
-   INFO(PACKAGE"-"VERSION"-"BUILDSTR" "UNAME" starting up");
+   INFO(PACKAGE"-"VERSION"-"BUILDSTR" "BUILDDATE" "UNAME" starting up");
 
    /* read the config file */
    if (read_config(configfile, config_search, main_cfg_opts, "") == STS_FAILURE) {
@@ -269,6 +281,7 @@ int main (int argc, char *argv[])
       setsid();
       if (fork()!=0) exit(0);
 
+      /* we are daemonizing, no more logging to stderr */
       log_set_stderr(0);
 
       /* detach STDIN/OUT/ERR file */
@@ -336,18 +349,14 @@ int main (int argc, char *argv[])
    /* initialize the registration facility */
    register_init();
 
-/*
- * silence the log - if so required...
- */
-   log_set_silence(configuration.silence_log);
-
-   INFO(PACKAGE"-"VERSION"-"BUILDSTR" "UNAME" started");
+   INFO(PACKAGE"-"VERSION"-"BUILDSTR" "BUILDDATE" "UNAME" started");
 
 /*****************************
  * Main loop
  *****************************/
    while (!exit_program) {
 
+      memset(&ticket, 0, sizeof(sip_ticket_t));
       while ((sts = sipsock_waitfordata(buff, sizeof(buff)-1,
                                     &ticket.from, &ticket.protocol)) <=0 ) {
 
@@ -386,6 +395,7 @@ int main (int argc, char *argv[])
       buflen = (size_t)sts;
       DEBUGC(DBCLASS_BABBLE,"received %zd bytes of data", buflen);
       ticket.direction=0;
+      ticket.timestamp=time(NULL);
       memset(&ticket.next_hop, 0, sizeof(ticket.next_hop));
       buff[buflen]='\0';
 
@@ -481,9 +491,19 @@ int main (int argc, char *argv[])
 
          DEBUGC(DBCLASS_PROXY,"checking Max-Forwards (=%i)",forwards_count);
          if (forwards_count <= 0) {
-            DEBUGC(DBCLASS_SIP, "Forward count reached 0 -> 483 response");
-            sip_gen_response(&ticket, 483 /*Too many hops*/);
-            goto end_loop; /* skip and free resources */
+            if (MSG_IS_REQUEST(ticket.sipmsg) && MSG_IS_OPTIONS(ticket.sipmsg)) {
+               // special treatment for an OPTIONS message with Max-Forwards=0
+               // -> RFC3261, 11.2 Processing of OPTIONS Request
+               //    and  16.3 Request Validation, step 3
+               // as this may be a request directed to us as proxy, reply to it.
+               DEBUGC(DBCLASS_SIP, "OPTION request with Max-Forwards=0 -> 200 response");
+               sip_gen_response(&ticket, 200);
+               goto end_loop; /* skip and free resources */
+            } else {
+               DEBUGC(DBCLASS_SIP, "Forward count reached 0 -> 483 response");
+               sip_gen_response(&ticket, 483 /*Too many hops*/);
+               goto end_loop; /* skip and free resources */
+            }
          }
       }
 
