@@ -18,7 +18,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */
 
-/* $Id: siproxd.h 524 2015-09-19 12:56:27Z hb9xar $ */
+/* $Id: siproxd.h 583 2019-11-09 17:41:08Z hb9xar $ */
 
 #ifdef DMALLOC
  #include <dmalloc.h>
@@ -125,9 +125,10 @@ typedef struct {
  */
 typedef struct {
    char *raw_buffer;		/* raw UDP packet */
-   size_t  raw_buffer_len;		/* length of raw data */
+   size_t  raw_buffer_len;	/* length of raw data */
    osip_message_t *sipmsg;	/* SIP */
    struct sockaddr_in from;	/* received from */
+   time_t timestamp;		/* timestamp of reception */
 #define PROTO_UNKN -1
 #define PROTO_UDP  1
 #define PROTO_TCP  2
@@ -236,8 +237,8 @@ int  rtp_start_fwd (osip_call_id_t *callid, client_id_t client_id,	/*X*/
                     int direction, int call_direction, int media_stream_no,
                     struct in_addr outbound_ipaddr, int *outboundport,
                     struct in_addr lcl_client_ipaddr, int lcl_clientport,
-                    int isrtp);
-int  rtp_stop_fwd (osip_call_id_t *callid, int direction);		/*X*/
+                    int isrtp, int cseq);
+int  rtp_stop_fwd (osip_call_id_t *callid, int direction, int cseq);	/*X*/
 
 /* accessctl.c */
 int  accesslist_check(struct sockaddr_in from);
@@ -281,13 +282,13 @@ int unload_plugins(void);
 #define TCP_IDLE_TO	300	/* TCP connection idle timeout in seconds */
 #define TCP_CONNECT_TO	500	/* TCP connect() timeout in msec */
 
-#define URLMAP_SIZE	128	/* number of URL mapping table entries	*/
+#define URLMAP_SIZE	512	/* number of URL mapping table entries	*/
 				/* this limits the number of clients!	*/
 
 #define SOURCECACHE_SIZE 256	/* number of return addresses		*/
 #define DEJITTERLIMIT	1500000	/* max value for dejitter configuration */
 
-#define RTPPROXY_SIZE	256	/* number of rtp proxy entries		*/
+#define RTPPROXY_SIZE	1024	/* number of rtp proxy entries		*/
 				/* this limits the number of calls!	*/
 
 #define BUFFER_SIZE	8196	/* input buffer for read from socket	*/
