@@ -50,7 +50,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: utils.c 536 2016-02-03 20:02:27Z hb9xar $";
+static char const ident[]="$Id: utils.c 585 2020-07-07 22:05:29Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
@@ -138,8 +138,10 @@ int get_ip_by_host(char *hostname, struct in_addr *addr) {
             idx=i;
             break;
          }
-         DEBUGC(DBCLASS_BABBLE, "DNS lookup - from cache: %s -> %s",
-                hostname, utils_inet_ntoa(*addr));
+// avoid: causes tremendous logging during URL lookups through
+//        the urlmap...
+//         DEBUGC(DBCLASS_BABBLE, "DNS lookup - from cache: %s -> %s",
+//                hostname, utils_inet_ntoa(*addr));
          return STS_SUCCESS;
       }
    }
